/*
 * file XmlNsVisitor.java
 *
 * IBM Confidential
 * OCO Source Materials
 * Rational ClearCase
 * (C) Copyright IBM Corp. 2010.  All rights reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.rational.wvcm.stpex.annotation.processing;

import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.INDENT;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.createFile;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.ensureUniqueAndAddIfSo;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.getRpcEnumValues;
import static com.ibm.rational.wvcm.stpex.annotation.processing.ListClassesVisitor.writeCommonHeader;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

import com.sun.mirror.apt.AnnotationProcessorEnvironment;
import com.sun.mirror.apt.Filer;
import com.sun.mirror.declaration.EnumConstantDeclaration;
import com.sun.mirror.declaration.FieldDeclaration;
import com.sun.mirror.util.SimpleDeclarationVisitor;

public class XmlNsVisitor extends SimpleDeclarationVisitor implements StpVisitor{
    private final Map<Integer, TagInfo> m_enums = new HashMap<Integer, TagInfo>();
    private final AnnotationProcessorEnvironment m_env;
    
    private static final String g_hxxFileName = "xmltag_ns_decl.hxx";
    private static final String g_cxxFileName = "xmltag_ns_impl.cxx";
    
    private PrintWriter m_hxxOut;
    private PrintWriter m_cxxOut;
    
    public XmlNsVisitor(AnnotationProcessorEnvironment env)
    {
        //System.out.println("Creating XmlNsVisitor");
        m_env = env;
    }
    
    public void begin() {
        //System.out.println("XmlNsVisitor begin");
        Filer filer = m_env.getFiler();
        writeHxxBegin(filer);
        writeCxxBegin(filer);
    }

    public void end() {
        
        TreeSet<Integer> keys = new TreeSet<Integer>(m_enums.keySet());
        writeHxxEnd(keys);
        writeCxxEnd(keys);
        //System.out.println("XmlNsVisitor end");
    }
    
    public void writeHxxBegin(Filer filer)
    {
        m_hxxOut = createFile(filer, g_hxxFileName);
        writeCommonHeader(m_hxxOut, this);
    }
    
    public void writeHxxEnd(TreeSet<Integer> keys)
    {
        for (Integer id : keys) {
            TagInfo info = m_enums.get(id);
            m_hxxOut.println(INDENT + info.fieldName + " = " + id + ",");
        }
        
        m_hxxOut.flush();
        m_hxxOut.close();        
    }
    
    public void writeCxxBegin(Filer filer) 
    {
        m_cxxOut = createFile(filer, g_cxxFileName);
        writeCommonHeader(m_cxxOut, this);
    }
    
    public void writeCxxEnd(TreeSet<Integer> keys)
    {
        for (Integer id : keys) {
            TagInfo info = m_enums.get(id);
            m_cxxOut.println("NS_DEFN(" + info.fieldName + ", L\"" + info.simple + "\", L\"" + info.ns + "\"),");  
        }
        
        m_cxxOut.flush();
        m_cxxOut.close();        
    }
    
    @Override
    public void visitFieldDeclaration(FieldDeclaration f)
    {
        TagInfo info = getRpcEnumValues(f.getAnnotationMirrors());
        if (info != null) {
            info.fieldName = f.getSimpleName();
            
            if (info.id == null || info.ns == null || info.simple == null) {
                
                System.err.print("Error: missing value for field '"+ info.fieldName + "' no values should be null: ");
                System.err.println("id = " + info.id + ", ns = " + info.ns + ", simple = " + info.simple);
                System.err.println("Error: exiting unexpectedly");
                System.exit(1);
            }
            
            ensureUniqueAndAddIfSo(m_enums, info);
        }
    }
}
